from appyter.fields import Field

class EmailField(Field):
  def __init__(self, **kwargs):
    super().__init__(**kwargs)

  def constraint(self):
    import re
    return self.raw_value is not None and re.match(r'\w[\w\d_-]+@\w[\w\d_-]+\.\w[\w\d_-]+', self.raw_value)



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../..'))

import sphinx
import commonmark
from m2r2 import MdInclude
from recommonmark.transform import AutoStructify
from textwrap import dedent

# -- Project information -----------------------------------------------------

project = "Appyter"
copyright = "2020, Ma'ayanlab"
author = "Ma'ayanlab"

# The full version, including alpha/beta/rc tags
release = '0.0.1'


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.

extensions = [
  'recommonmark',
  'sphinx.ext.autodoc',
  'sphinx.ext.viewcode',
  'sphinx.ext.autosectionlabel',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'nature'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

autosectionlabel_prefix_document = True

class CustomReStructuredTextRenderer(commonmark.ReStructuredTextRenderer):
    def code_block(self, node, entering):
        language_name = None

        info_words = node.info.split() if node.info else []
        if len(info_words) > 0 and len(info_words[0]) > 0:
            language_name = info_words[0]

        if language_name == 'eval_rst':
            self.out(dedent(node.literal))
        else:
            super().code_block(node, entering)

def docstring(app, what, name, obj, options, lines):
    md  = '\n'.join(lines)
    ast = commonmark.Parser().parse(md)
    rst = CustomReStructuredTextRenderer().render(ast)
    lines.clear()
    lines += rst.splitlines()

def setup(app):
    config = {
        # 'url_resolver': lambda url: github_doc_root + url,
        'auto_toc_tree_section': 'Contents',
        'enable_eval_rst': True,
    }
    app.add_config_value('recommonmark_config', config, True)
    app.add_transform(AutoStructify)
    app.connect('autodoc-process-docstring', docstring)
    app.add_config_value('no_underscore_emphasis', False, 'env')
    app.add_config_value('m2r_parse_relative_links', False, 'env')
    app.add_config_value('m2r_anonymous_references', False, 'env')
    app.add_config_value('m2r_disable_inline_math', False, 'env')
    app.add_config_value('m2r_use_mermaid', False, 'env')
    app.add_directive('mdinclude', MdInclude)



def testdash(app, url_prefix='/testdash', DATA_DIR=''):
  import pandas as pd
  import dash
  from dash.dependencies import Input, Output
  import dash_core_components as dcc
  import dash_html_components as html

  df = pd.read_csv('https://raw.githubusercontent.com/plotly/datasets/master/dash-stock-ticker-demo.csv')

  testdash = dash.Dash(
    'testdash',
    server=app,
    routes_pathname_prefix=url_prefix + '/',
  )

  testdash.layout = html.Div([
      html.H1('Stock Tickers'),
      dcc.Dropdown(
          id='my-dropdown',
          options=[
              {'label': 'Tesla', 'value': 'TSLA'},
              {'label': 'Apple', 'value': 'AAPL'},
              {'label': 'Coke', 'value': 'COKE'}
          ],
          value='TSLA'
      ),
      dcc.Graph(id='my-graph')
  ], className="container")

  @testdash.callback(Output('my-graph', 'figure'),
                [Input('my-dropdown', 'value')])
  def update_graph(selected_dropdown_value):
      dff = df[df['Stock'] == selected_dropdown_value]
      return {
          'data': [{
              'x': dff.Date,
              'y': dff.Close,
              'line': {
                  'width': 3,
                  'shape': 'spline'
              }
          }],
          'layout': {
              'margin': {
                  'l': 30,
                  'r': 20,
                  'b': 30,
                  't': 20
              }
          }
      }
